@@ -1,20 +1,18 @@
 use crate::diff;
-use crate::matcher::{Matcher, PathAndQueryMatcher};
-use crate::response::{Body, Response};
+use crate::matcher::{Matcher, PathAndQueryMatcher, RequestMatcher};
+use crate::response::{Body, Header, Response, ResponseStatusCode};
 use crate::server::RemoteMock;
 use crate::server::State;
 use crate::Request;
 use crate::{Error, ErrorKind};
 use bytes::Bytes;
 use http::{HeaderMap, HeaderName, StatusCode};
-use rand::distributions::Alphanumeric;
-use rand::{thread_rng, Rng};
-use std::convert::Into;
+use rand::distr::Alphanumeric;
+use rand::Rng;
 use std::fmt;
 use std::io;
 use std::ops::Drop;
 use std::path::Path;
-use std::string::ToString;
 use std::sync::Arc;
 use std::sync::RwLock;
 
@@ -67,6 +65,7 @@ pub struct InnerMock {
     pub(crate) path: PathAndQueryMatcher,
     pub(crate) headers: HeaderMap<Matcher>,
     pub(crate) body: Matcher,
+    pub(crate) request_matcher: RequestMatcher,
     pub(crate) response: Response,
     pub(crate) hits: usize,
     pub(crate) expected_hits_at_least: Option<usize>,
@@ -152,7 +151,7 @@ impl Mock {
         assert_on_drop: bool,
     ) -> Mock {
         let inner = InnerMock {
-            id: thread_rng()
+            id: rand::rng()
                 .sample_iter(&Alphanumeric)
                 .map(char::from)
                 .take(24)
@@ -161,6 +160,7 @@ impl Mock {
             path: PathAndQueryMatcher::Unified(path.into()),
             headers: HeaderMap::<Matcher>::default(),
             body: Matcher::Any,
+            request_matcher: RequestMatcher::default(),
             response: Response::default(),
             hits: 0,
             expected_hits_at_least: None,
@@ -303,6 +303,36 @@ impl Mock {
         self
     }
 
+    ///
+    /// Allows matching the entire request based on a closure that takes
+    /// the [`Request`] object as an argument and returns a boolean value.
+    ///
+    /// ## Example
+    ///
+    /// ```
+    /// use mockito::Matcher;
+    ///
+    /// let mut s = mockito::Server::new();
+    ///
+    /// // This will match requests that have the x-test header set
+    /// // and contain the word "hello" inside the body
+    /// s.mock("GET", "/")
+    ///     .match_request(|request| {
+    ///         request.has_header("x-test") &&
+    ///             request.utf8_lossy_body().unwrap().contains("hello")
+    ///     })
+    ///     .create();
+    /// ```
+    ///
+    pub fn match_request<F>(mut self, request_matcher: F) -> Self
+    where
+        F: Fn(&Request) -> bool + Send + Sync + 'static,
+    {
+        self.inner.request_matcher = request_matcher.into();
+
+        self
+    }
+
     ///
     /// Sets the status code of the mock response. The default status code is 200.
     ///
@@ -316,10 +346,45 @@ impl Mock {
     ///
     #[track_caller]
     pub fn with_status(mut self, status: usize) -> Self {
-        self.inner.response.status = StatusCode::from_u16(status as u16)
-            .map_err(|_| Error::new_with_context(ErrorKind::InvalidStatusCode, status))
-            .unwrap();
+        self.inner.response.status = ResponseStatusCode::StatusCode(
+            StatusCode::from_u16(status as u16)
+                .map_err(|_| Error::new_with_context(ErrorKind::InvalidStatusCode, status))
+                .unwrap(),
+        );
+        self
+    }
 
+    ///
+    /// Sets the status code of the mock response dynamically while exposing the request object.
+    ///
+    /// You can use this method to provide custom status code for every incoming request.
+    ///
+    /// The function must be thread-safe. If it's a closure, it can't be borrowing its context.
+    /// Use `move` closures and `Arc` to share any data.
+    ///
+    /// ### Example
+    ///
+    /// ```
+    /// let mut s = mockito::Server::new();
+    ///
+    /// let _m = s.mock("GET", mockito::Matcher::Any).with_status_code_from_request(|request| {
+    ///     if request.path() == "/bob" {
+    ///         500
+    ///     } else if request.path() == "/alice" {
+    ///         400
+    ///     } else {
+    ///         404
+    ///     }
+    /// });
+    /// ```
+    ///
+    #[track_caller]
+    pub fn with_status_code_from_request(
+        mut self,
+        callback: impl Fn(&Request) -> usize + Send + Sync + 'static,
+    ) -> Self {
+        self.inner.response.status =
+            ResponseStatusCode::FnWithRequest(Arc::new(move |req| callback(req)));
         self
     }
 
@@ -338,11 +403,47 @@ impl Mock {
         self.inner
             .response
             .headers
-            .append(field.into_header_name(), value.to_owned());
+            .append(field.into_header_name(), Header::String(value.to_string()));
 
         self
     }
 
+    ///
+    /// Sets the headers of the mock response dynamically while exposing the request object.
+    ///
+    /// You can use this method to provide custom headers for every incoming request.
+    ///
+    /// The function must be thread-safe. If it's a closure, it can't be borrowing its context.
+    /// Use `move` closures and `Arc` to share any data.
+    ///
+    /// ### Example
+    ///
+    /// ```
+    /// let mut s = mockito::Server::new();
+    ///
+    /// let _m = s.mock("GET", mockito::Matcher::Any).with_header_from_request("x-user", |request| {
+    ///     if request.path() == "/bob" {
+    ///         "bob".into()
+    ///     } else if request.path() == "/alice" {
+    ///         "alice".into()
+    ///     } else {
+    ///         "everyone".into()
+    ///     }
+    /// });
+    /// ```
+    ///
+    pub fn with_header_from_request<T: IntoHeaderName>(
+        mut self,
+        field: T,
+        callback: impl Fn(&Request) -> String + Send + Sync + 'static,
+    ) -> Self {
+        self.inner.response.headers.append(
+            field.into_header_name(),
+            Header::FnWithRequest(Arc::new(move |req| callback(req))),
+        );
+        self
+    }
+
     ///
     /// Sets the body of the mock response. Its `Content-Length` is handled automatically.
     ///
