@@ -2,7 +2,7 @@
 extern crate serde_json;
 
 use mockito::{Matcher, Server, ServerOpts};
-use rand::distributions::Alphanumeric;
+use rand::distr::Alphanumeric;
 use rand::Rng;
 use std::fmt::Display;
 use std::fs;
@@ -587,6 +587,20 @@ fn test_mock_with_custom_status() {
     assert_eq!("HTTP/1.1 499 <none>\r\n", status_line);
 }
 
+#[test]
+fn test_mock_with_status_code_from_request() {
+    let mut s = Server::new();
+    s.mock("GET", Matcher::Any)
+        .with_status_code_from_request(|request| if request.path() == "/world" { 500 } else { 404 })
+        .create();
+
+    let (status_line, _, _) = request(s.host_with_port(), "GET /world", "");
+    assert_eq!("HTTP/1.1 500 Internal Server Error\r\n", status_line);
+
+    let (status_line, _, _) = request(s.host_with_port(), "GET /", "");
+    assert_eq!("HTTP/1.1 404 Not Found\r\n", status_line);
+}
+
 #[test]
 fn test_mock_with_body() {
     let mut s = Server::new();
@@ -677,6 +691,25 @@ fn test_mock_with_header() {
     assert!(headers.contains(&"content-type: application/json".to_string()));
 }
 
+#[test]
+fn test_mock_with_header_from_request() {
+    let mut s = Server::new();
+    s.mock("GET", Matcher::Any)
+        .with_header_from_request("x-user", |req| {
+            if req.path() == "/alice" {
+                "alice".into()
+            } else {
+                "everyone".into()
+            }
+        })
+        .create();
+
+    let (_, headers, _) = request(s.host_with_port(), "GET /alice", "");
+    assert!(headers.contains(&"x-user: alice".to_string()));
+    let (_, headers, _) = request(s.host_with_port(), "GET /anyone-else", "");
+    assert!(headers.contains(&"x-user: everyone".to_string()));
+}
+
 #[test]
 fn test_mock_with_multiple_headers() {
     let mut s = Server::new();
@@ -998,7 +1031,7 @@ fn test_all_of_missing_match_header() {
 #[test]
 fn test_large_utf8_body() {
     let mut s = Server::new();
-    let mock_body: String = rand::thread_rng()
+    let mock_body: String = rand::rng()
         .sample_iter(&Alphanumeric)
         .map(char::from)
         .take(3 * 1024) // Must be larger than the request read buffer
@@ -1879,6 +1912,69 @@ fn test_anyof_exact_path_and_query_matcher() {
     mock.assert();
 }
 
+#[test]
+fn test_request_matcher_path() {
+    let mut s = Server::new();
+    let host = s.host_with_port();
+    let m = s
+        .mock("GET", Matcher::Any)
+        .match_request(|req| req.path().contains("hello"))
+        .with_body("world")
+        .create();
+
+    let (status_line, _, _) = request(&host, "GET /", "");
+    assert_eq!("HTTP/1.1 501 Not Implemented\r\n", status_line);
+
+    let (status_line, _, body) = request(host, "GET /hello", "");
+    assert_eq!("HTTP/1.1 200 OK\r\n", status_line);
+    assert_eq!("world", body);
+
+    m.assert();
+}
+
+#[test]
+fn test_request_matcher_headers() {
+    let mut s = Server::new();
+    let host = s.host_with_port();
+    let m = s
+        .mock("GET", "/")
+        .match_request(|req| req.has_header("x-test"))
+        .with_body("world")
+        .create();
+
+    let (status_line, _, _) = request(&host, "GET /", "");
+    assert_eq!("HTTP/1.1 501 Not Implemented\r\n", status_line);
+
+    let (status_line, _, body) = request(host, "GET /", "x-test: 1\r\n");
+    assert_eq!("HTTP/1.1 200 OK\r\n", status_line);
+    assert_eq!("world", body);
+
+    m.assert();
+}
+
+#[test]
+fn test_request_matcher_body() {
+    let mut s = Server::new();
+    let host = s.host_with_port();
+    let m = s
+        .mock("GET", "/")
+        .match_request(|req| {
+            let body = req.utf8_lossy_body().unwrap();
+            body.contains("hello")
+        })
+        .with_body("world")
+        .create();
+
+    let (status_line, _, _) = request_with_body(&host, "GET /", "", "bye");
+    assert_eq!("HTTP/1.1 501 Not Implemented\r\n", status_line);
+
+    let (status_line, _, body) = request_with_body(host, "GET /", "", "hello");
+    assert_eq!("HTTP/1.1 200 OK\r\n", status_line);
+    assert_eq!("world", body);
+
+    m.assert();
+}
+
 #[test]
 fn test_default_headers() {
     let mut s = Server::new();
@@ -2176,6 +2272,42 @@ async fn test_match_body_asnyc() {
     assert_eq!(200, response.status());
 }
 
+#[tokio::test]
+async fn test_request_matcher_body_async() {
+    let mut s = Server::new_async().await;
+    let m = s
+        .mock("POST", "/")
+        .match_request(|req| {
+            let body = req.utf8_lossy_body().unwrap();
+            body.contains("hello")
+        })
+        .with_body("world")
+        .create_async()
+        .await;
+
+    let response = reqwest::Client::new()
+        .post(s.url())
+        .version(reqwest::Version::HTTP_11)
+        .body("bye")
+        .send()
+        .await
+        .unwrap();
+
+    assert_eq!(501, response.status());
+
+    let response = reqwest::Client::new()
+        .post(s.url())
+        .version(reqwest::Version::HTTP_11)
+        .body("hello")
+        .send()
+        .await
+        .unwrap();
+
+    assert_eq!(200, response.status());
+
+    m.assert_async().await;
+}
+
 #[tokio::test(flavor = "multi_thread")]
 async fn test_join_all_async() {
     let _lock = SERIAL_POOL_TESTS.lock().await;
